@@ -22,6 +22,7 @@ from ansiblelint._internal.rules import (
     RuntimeErrorRule,
     WarningRule,
 )
+from ansiblelint.app import App, get_app
 from ansiblelint.config import PROFILES, Options, get_rule_config
 from ansiblelint.config import options as default_options
 from ansiblelint.constants import LINE_NUMBER_KEY, RULE_DOC_URL, SKIPPED_RULES_KEY
@@ -31,6 +32,8 @@ from ansiblelint.file_utils import Lintable, expand_paths_vars
 if TYPE_CHECKING:
     from ruamel.yaml.comments import CommentedMap, CommentedSeq
 
+    from ansiblelint.errors import RuleMatchTransformMeta
+
 _logger = logging.getLogger(__name__)
 
 match_types = {
@@ -77,6 +80,7 @@ class AnsibleLintRule(BaseRule):
         details: str = "",
         filename: Lintable | None = None,
         tag: str = "",
+        transform_meta: RuleMatchTransformMeta | None = None,
     ) -> MatchError:
         """Instantiate a new MatchError."""
         match = MatchError(
@@ -86,6 +90,7 @@ class AnsibleLintRule(BaseRule):
             lintable=filename or Lintable(""),
             rule=copy.copy(self),
             tag=tag,
+            transform_meta=transform_meta,
         )
         # search through callers to find one of the match* methods
         frame = inspect.currentframe()
@@ -141,8 +146,7 @@ class AnsibleLintRule(BaseRule):
             matches.append(matcherror)
         return matches
 
-    # pylint: disable=too-many-branches
-    def matchtasks(self, file: Lintable) -> list[MatchError]:  # noqa: C901
+    def matchtasks(self, file: Lintable) -> list[MatchError]:
         """Call matchtask for each task inside file and return aggregate results.
 
         Most rules will never need to override matchtasks because its main
@@ -325,7 +329,7 @@ class TransformMixin:
 
 
 # pylint: disable=too-many-nested-blocks
-def load_plugins(  # : max-complexity: 11
+def load_plugins(
     dirs: list[str],
 ) -> Iterator[AnsibleLintRule]:
     """Yield a rule class."""
@@ -370,17 +374,26 @@ def load_plugins(  # : max-complexity: 11
 class RulesCollection:
     """Container for a collection of rules."""
 
-    def __init__(
+    def __init__(  # pylint: disable=too-many-arguments
         self,
         rulesdirs: list[str] | list[Path] | None = None,
-        options: Options = default_options,
+        options: Options | None = None,
         profile_name: str | None = None,
         *,
         conditional: bool = True,
+        app: App | None = None,
     ) -> None:
         """Initialize a RulesCollection instance."""
-        self.options = options
+        if options is None:
+            self.options = copy.deepcopy(default_options)
+            # When initialized without options argument we want it to always
+            # be offline as this is done only during testing.
+            self.options.offline = True
+        else:
+            self.options = options
         self.profile = []
+        self.app = app or get_app(offline=True)
+
         if profile_name:
             self.profile = PROFILES[profile_name]
         rulesdirs_str = [] if rulesdirs is None else [str(r) for r in rulesdirs]
@@ -409,6 +422,7 @@ class RulesCollection:
         """Register a rule."""
         # We skip opt-in rules which were not manually enabled.
         # But we do include opt-in rules when listing all rules or tags
+        obj._collection = self  # pylint: disable=protected-access # noqa: SLF001
         if any(
             [
                 not conditional,
@@ -419,7 +433,6 @@ class RulesCollection:
                 self.options.list_tags,
             ],
         ):
-            obj._collection = self  # pylint: disable=protected-access # noqa: SLF001
             self.rules.append(obj)
 
     def __iter__(self) -> Iterator[BaseRule]:
@@ -438,7 +451,7 @@ class RulesCollection:
         """Combine rules."""
         self.rules.extend(more)
 
-    def run(  # : max-complexity: 12
+    def run(
         self,
         file: Lintable,
         tags: set[str] | None = None,
@@ -503,6 +516,7 @@ class RulesCollection:
             "metadata": "Invalid metadata, likely related to galaxy, collections or roles",
             "opt-in": "Rules that are not used unless manually added to `enable_list`",
             "security": "Rules related o potentially security issues, like exposing credentials",
+            "syntax": "Related to wrong or deprecated syntax",
             "unpredictability": "Warn about code that might not work in a predictable way",
             "unskippable": "Indicate a fatal error that cannot be ignored or disabled",
             "yaml": "External linter which will also produce its own rule codes",
@@ -510,8 +524,12 @@ class RulesCollection:
 
         tags = defaultdict(list)
         for rule in self.rules:
+            # Fail early if a rule does not have any of our required tags
+            if not set(rule.tags).intersection(tag_desc.keys()):
+                msg = f"Rule {rule} does not have any of the required tags: {', '.join(tag_desc.keys())}"
+                raise RuntimeError(msg)
             for tag in rule.tags:
-                tags[tag].append(rule.id)
+                tags[tag] = list(rule.ids())
         result = "# List of tags and rules they cover\n"
         for tag in sorted(tags):
             desc = tag_desc.get(tag, None)
@@ -519,7 +537,7 @@ class RulesCollection:
                 result += f"{tag}:  # {desc}\n"
             else:
                 result += f"{tag}:\n"
-            for name in tags[tag]:
+            for name in sorted(tags[tag]):
                 result += f"  - {name}\n"
         return result
 
