@@ -16,6 +16,7 @@ from ansiblelint.file_utils import Lintable
 from ansiblelint.rules import AnsibleLintRule, RulesCollection
 from ansiblelint.runner import Runner
 from ansiblelint.skip_utils import get_rule_skips_from_line
+from ansiblelint.text import has_jinja, is_fqcn_or_name
 from ansiblelint.utils import parse_yaml_from_file
 
 if TYPE_CHECKING:
@@ -160,10 +161,15 @@ class VariableNamingRule(AnsibleLintRule):
                 rule=self,
             )
 
-        if prefix and not ident.startswith(f"{prefix}_"):
+        if (
+            prefix
+            and not ident.startswith(f"{prefix}_")
+            and not has_jinja(prefix)
+            and is_fqcn_or_name(prefix)
+        ):
             return MatchError(
                 tag="var-naming[no-role-prefix]",
-                message="Variables names from within roles should use role_name_ as a prefix.",
+                message=f"Variables names from within roles should use {prefix}_ as a prefix.",
                 rule=self,
             )
         return None
@@ -219,6 +225,8 @@ class VariableNamingRule(AnsibleLintRule):
             if isinstance(action, dict):
                 role_fqcn = action.get("name", "")
                 prefix = role_fqcn.split("/" if "/" in role_fqcn else ".")[-1]
+        else:
+            prefix = ""
         for key in our_vars:
             match_error = self.get_var_naming_matcherror(key, prefix=prefix)
             if match_error:
@@ -296,7 +304,11 @@ if "pytest" in sys.modules:
     @pytest.mark.parametrize(
         ("file", "expected"),
         (
-            pytest.param("examples/playbooks/rule-var-naming-fail.yml", 7, id="0"),
+            pytest.param(
+                "examples/playbooks/var-naming/rule-var-naming-fail.yml",
+                7,
+                id="0",
+            ),
             pytest.param("examples/Taskfile.yml", 0, id="1"),
         ),
     )
@@ -362,7 +374,7 @@ if "pytest" in sys.modules:
 
     def test_var_naming_with_include_role_import_role() -> None:
         """Test with include role and import role."""
-        role_path = "examples/test_collection/roles/my_role/tasks/main.yml"
+        role_path = "examples/.test_collection/roles/my_role/tasks/main.yml"
         result = run_ansible_lint(role_path)
         assert result.returncode == RC.SUCCESS
         assert "var-naming" not in result.stdout
